/* global importScripts */
/* global xGraph */
/* global xAstar */

importScripts("astar.js");

var currentCollisionMap;
var collisionQueue = [];
var collissionPending = false;
var recycleBin = [];

var pendingIncomingMap = null;

self.addEventListener("message",function(e) {

    var msg = e.data;
    var useCr;

    if(!msg) {
        return;
    }

    if(msg.method == "updateCollisionMap") {

        if(collissionPending) {
            pendingIncomingMap = msg.data;
        } else {
            currentCollisionMap = null;
            currentCollisionMap = new xGraph(msg.data);
            pendingIncomingMap = null;
        }

        

        processCollissions();
    }

    if(msg.method == "checkPath") {

        useCr = null;

        if(msg.data.cr && msg.data.cr != null) {
            useCr = msg.data.cr;
        }

        var ci = getCollissionItem(msg.data.x1,msg.data.y1,msg.data.x2,msg.data.y2,msg.data.type,msg.data.id,useCr);
        collisionQueue.push([ci]);
        processCollissions();
    }

    if(msg.method == "checkPaths") {
        var pathData = [];

        useCr = null;
        
        if(msg.data.cr && msg.data.cr != null) {
            useCr = msg.data.cr;
        }

        while(msg.data.length > 0) {
            var pth = msg.data.shift();

            var cp = getCollissionItem(pth.x1,pth.y1,pth.x2,pth.y2,pth.type,pth.id,useCr);
            pathData.push(cp);
        }


        collisionQueue.push(pathData);
        processCollissions();
    }
});

function CollisionItem(x1,y1,x2,y2,type,id,cr) {
    this.x1 = x1;
    this.y1 = y1;
    this.x2 = x2;
    this.y2 = y2;
    this.type = type;
    this.id = id;
    this.cr = cr;
}

function getCollissionItem(x1,y1,x2,y2,type,id,cr) {
    if(recycleBin.length == 0) {
        return new CollisionItem(x1,y1,x2,y2,type,id,cr);
    } else {
        var ti = recycleBin.shift();
        ti.x1 = x1;
        ti.y1 = y1;
        ti.x2 = x2;
        ti.y2 = y2;
        ti.type = type;
        ti.id = id;
        ti.cr = cr;
        return ti;
    }
}

function processCollissions() {
    if(collisionQueue.length == 0) {
        return;
    }

    if(collissionPending) {
        return;
    }

    collissionPending = true;

    var workingItems = collisionQueue.shift();

    var pth = [];
    var workingItem = null;

    while(workingItems.length > 0) {
        workingItem = workingItems.shift();
        pth = getPath(workingItem.x1,workingItem.y1,workingItem.x2,workingItem.y2,workingItem.cr);

        if(pth.length > 0) {
            break;
        }
    }

    

    if(workingItem == null) {
        workMapIfNeeded();
        collissionPending = false;
        workingItems = null;
        processCollissions();
        return;
    } else {
        reportCollissionResult(pth,workingItem);
    }

    workingItems = null;
}

function getPath(x1,y1,x2,y2,cr) {

    var useMap = currentCollisionMap;

    if(cr && cr != null) {
        useMap = new xGraph(cr);
    }

    if(!useMap) {
        return [];
    }

    if(useMap.grid[x1] == undefined || useMap.grid[x2] == undefined) {
        return [];
    }

    // is this reachable
    var start = useMap.grid[x1][y1];
    var end = useMap.grid[x2][y2];

    if(!start || !end) {
        return [];
    }

    return xAstar.search(useMap,start,end);
}

function reportCollissionResult(data,workingItem) {

    var clean = [];

    while(data.length > 0) {
        clean.push(new CleanNode(data.shift()));
    }

    data = null;

    self.postMessage({"method": "checkPath", "data": {
        "data": clean,
        "type": workingItem.type,
        "id": workingItem.id
    } });

    recycleBin.push(workingItem);
    workMapIfNeeded();
    collissionPending = false;
    processCollissions();
}

function workMapIfNeeded() {
    if(pendingIncomingMap != null) {
        currentCollisionMap = null;
        currentCollisionMap = new xGraph(pendingIncomingMap);
        pendingIncomingMap = null;
    }
}

function CleanNode(old) {
    this.x = old.x;
    this.y = old.y;
    old = null;
}