/* global webAppCore */
/* global ColonyGame */
(function() {

    window.wacMenuOptionSelected = onMenuOptionSelected;

    window.addEventListener("load",startApp);

    var currentModFile = null;
    var currentFileName = null;
    var pendingOpenFileName = null;

    function startApp() {
        webAppCore.startApp(initPage);
    }

    function initPage() {

        currentModFile = new TextureModData();

        linkLayouts();
        setupMenuOptions();

        populateTextureList(true);
        populateMetaData();

        var lastSaveData = webAppCore.getVar("text-edit-last-save","0");

        if(lastSaveData == "0") {
            webAppCore.clearSplash();
        } else {
            pendingOpenFileName = lastSaveData;
            webAppCore.openFile(lastSaveData,onFileOpened);
        }
    }

    function TextureModData() {
        this.meta = {
            "name": "",
            "revision": 0,
            "author": "",
            "description": "",
            "url": "",
            "serverId": 0
        };

        this.overrides = {};
    }

    function linkLayouts() {
        document.getElementById("btnOptions").addEventListener("click",function() {
            webAppCore.toggleMenu();
        });

        var metaButtons = document.querySelectorAll("[data-meta-item]");

        for(var i = 0; i < metaButtons.length; i++) {
            metaButtons[i].addEventListener("change",function() {
                var prop = this.dataset.metaItem;
                currentModFile.meta[prop] = this.value.latinize();
            });
        }

        document.getElementById("overrideImportImage").onclick = function() {
            webAppCore.importImage(onImageImport);
        };

        document.getElementById("overrideImportSvg").onclick = function() {
            webAppCore.importFile("svg",onSvgImport);
        };

        document.getElementById("clearOverride").onclick = function() {
            if(currentAssetItem == null) {
                return;
            }

            delete currentModFile.overrides[currentAssetItem.name];

            loadOverrideForCurrent();
            populateTextureList(false);
        };

        if(!ColonyGame.meta.colonyServerApi) {
            document.getElementById("btnUploadMod").style.display = "none";
        } else {

            //////  TEMP FOR NOW!
            //document.getElementById("btnUploadMod").style.display = "none";

            document.getElementById("btnUploadMod").onclick = function() {
                if(webAppCore.isApeAppsLoggedIn()) {

                    webAppCore.showDialog({
                        "title":    "Upload Mod",
                        "message":  "By uploading your mod to the Ape Apps servers, you agree that your mod contains no copyrighted content and to allow Ape Apps to distribute your mod to other players.",
                        "op1": {
                            "text":     "Agree",
                            "func":     doModUpload,
                            "color":    "#7CB342"
                        },
                        "op2": {
                            "text":     "Nevermind",
                            "func":     placeholder,
                            "color":    "#F44336"
                        }
                    });
                } else {
                    webAppCore.showAlertWithTitle("You must be signed in to your Ape Apps Accounts to upload mods.","Ape Apps Account");
                }
            };
        }

        
    }

    function registerModServerId() {
        if(!webAppCore.isApeAppsLoggedIn()) {
            return;
        }

        var keys = webAppCore.getApeAppsLoginKeys();
        var username = webAppCore.getApeAppsProfile().un;

        if(!keys) {
            return;
        }

        var data = [];

        data.push({
            "key":      "f",
            "value":    44
        });

        data.push({
            "key":      "g",
            "value":    ColonyGame.meta.colonyServerGameId
        });

        data.push({
            "key":      "aun",
            "value":    username
        });

        data.push({
            "key":      "sa",
            "value":    keys.a
        });

        data.push({
            "key":      "sb",
            "value":    keys.b
        });

        data.push({
            "key":      "sc",
            "value":    keys.c
        });

        data.push({
            "key":      "relcd",
            "value":    ColonyGame.meta.releaseCode
        });



        webAppCore.postForResult(ColonyGame.meta.colonyServerApi, data, onServerIdRegistration);
    }

    function onServerIdRegistration(data) {
        console.log(data);

        if(data && data.trim().length > 0 && data.indexOf("invalid") == -1) {
            currentModFile.meta.serverId = parseInt(data.trim());
            saveMod(doModUpload);
        }
    }

    function doModUpload() {
        if(!webAppCore.isApeAppsLoggedIn()) {
            return;
        }

        if(!currentModFile.meta.serverId || currentModFile.meta.serverId == 0) {
            registerModServerId();
            return;
        }

        var keys = webAppCore.getApeAppsLoginKeys();
        var username = webAppCore.getApeAppsProfile().un;

        if(!keys) {
            return;
        }

        currentModFile.meta.author = webAppCore.getApeAppsProfile().un;
        var saveContent = JSON.stringify(currentModFile);
        var uploadContent = btoa(saveContent);

        var data = [];

        data.push({
            "key":      "f",
            "value":    45
        });

        data.push({
            "key":      "g",
            "value":    ColonyGame.meta.colonyServerGameId
        });

        data.push({
            "key":      "aun",
            "value":    username
        });

        data.push({
            "key":      "sa",
            "value":    keys.a
        });

        data.push({
            "key":      "sb",
            "value":    keys.b
        });

        data.push({
            "key":      "sc",
            "value":    keys.c
        });

        data.push({
            "key":      "mod",
            "value":    uploadContent
        });

        data.push({
            "key":      "relcd",
            "value":    ColonyGame.meta.releaseCode
        });

        webAppCore.postForResult(ColonyGame.meta.colonyServerApi, data, onModUploadComplete);
    }

    function onModUploadComplete(data) {
        if(data) {
            webAppCore.showToast(data);
        }
    }

    function setupMenuOptions() {
        webAppCore.clearMenu();

        webAppCore.addMenuOption("Close Editor","wacMenuOptionSelected","back","#F44336","closeeditor");

        webAppCore.addMenuOption("New","wacMenuOptionSelected","new",null,"new");

        webAppCore.addMenuOption("Save","wacMenuOptionSelected","save",null,"save");

        webAppCore.addMenuOption("Open","wacMenuOptionSelected","openfile",null,"open");

        webAppCore.addMenuOption("Export","wacMenuOptionSelected","savelocal",null,"export");
    }

    function startNewModFile() {
        webAppCore.storeVar("text-edit-last-save","0");
        currentModFile = null;
        currentFileName = null;
        pendingOpenFileName = null;
        currentModFile = new TextureModData();
        populateTextureList(true);
        populateMetaData();
    }

    function onMenuOptionSelected(name,tag) {

        if(tag == "new") {
            
            startNewModFile();
        }

        if(tag == "closeeditor") {
            if(currentFileName == null) {
                var moditems = 0;

                for(var prop in currentModFile.overrides) {
                    console.log(prop);
                    moditems++;
                }

                if(moditems > 0) {
                    webAppCore.showDialog({
                        "title":    "Unsaved Changes",
                        "message":  "There are unsaved changes on this mod.  Do you want to save your work before leaving?",
                        "op1": {
                            "text":     "Save",
                            "func":     function() {
                                saveMod(closeEditor);
                            }
                        },
                        "op2": {
                            "text":     "Quit",
                            "func":     closeEditor
                        }
                    });
                } else {
                    closeEditor();
                }
            } else {
                saveMod(closeEditor);
            }
        }

        if(tag == "save") {
            saveMod(placeholder);
            populateMetaData();
        }

        if(tag == "open") {
            webAppCore.openListing(onListingBack);
        }

        if(tag == "export") {
            saveMod(function() {
                webAppCore.exportFile(currentFileName,JSON.stringify(currentModFile),false);
                populateMetaData();
            });
        }
    }

    function onListingBack(data) {

        var menuOptions = [];

        for(var i = 0; i < data.length; i++) {
            if(endsWith(data[i],"." + ColonyGame.meta.graphicsModFileFormat)) {
                menuOptions.push({
                    "title": data[i],
                    "tag": data[i]
                });
            }
        }

        if(menuOptions.length == 0) {
            webAppCore.showToast("No installed mods!");
        } else {
            webAppCore.presentNativeList("Open",menuOptions,function(tag) {

                if(!tag) {
                    return;
                }

                pendingOpenFileName = tag;
                webAppCore.openFile(tag,onFileOpened);
            },webAppCore.getString("general-dismiss"));
        }
    }

    function endsWith(str, suffix) {
        
        if(!str || !suffix) {
            return false;
        }

        return str.indexOf(suffix, str.length - suffix.length) !== -1;
    }

    function closeEditor() {
        webAppCore.showSplash("Saving mod...");
        webAppCore.goBack();
    }

    function populateTextureList(loadFirst) {
        document.getElementById("textureListing").innerHTML = "";

        for(var i = 0; i < ColonyGame.tiles.length; i++) {
            var sisebarItem = document.createElement("p");
            sisebarItem.innerHTML = ColonyGame.tiles[i].name;
            sisebarItem.textureReference = ColonyGame.tiles[i];
            sisebarItem.className = "sidebarItem";

            if(currentModFile.overrides[ColonyGame.tiles[i].name]) {
                sisebarItem.className += " override";
            }

            sisebarItem.addEventListener("click",function() {
                loadTexture(this.textureReference);
            });

            document.getElementById("textureListing").appendChild(sisebarItem);

            if(loadFirst && i == 0) {
                loadTexture(ColonyGame.tiles[i]);
            }
        }
    }

    function populateMetaData() {
        var metaButtons = document.querySelectorAll("[data-meta-item]");

        for(var i = 0; i < metaButtons.length; i++) {
            metaButtons[i].value = currentModFile.meta[metaButtons[i].dataset.metaItem];

            if(metaButtons[i].dataset.metaItem == "author" && webAppCore.isApeAppsLoggedIn()) {
                metaButtons[i].disabled = true;
                metaButtons[i].value = webAppCore.getApeAppsProfile().un;
            }
        }
    }

    var currentOriginal = null;
    var currentAssetItem = null;
    var currentOverride = null;

    function loadTexture(texture) {
        currentAssetItem = texture;

        document.getElementById("curTexName").innerHTML = currentAssetItem.name;
        document.getElementById("curTexWidth").innerHTML = "Frame Width: " + currentAssetItem.tileWidth + "px";

        currentOriginal = new Image();
        currentOriginal.onload = function() {
            var frames = this.width / currentAssetItem.tileWidth;
            document.getElementById("origFrameCount").innerHTML = "Frames: " + frames;
        };
        currentOriginal.src = "images/" + currentAssetItem.image;

        document.getElementById("originalImage").src = "images/" + currentAssetItem.image;

        document.getElementById("origExportImage").onclick = function() {
            var exCanvas = document.createElement("canvas");
            exCanvas.style.backgroundColor = "transparent";
            document.body.appendChild(exCanvas);
            exCanvas.width = currentOriginal.width;
            exCanvas.height = currentOriginal.height;
            var exContext = exCanvas.getContext("2d");
            exContext.drawImage(currentOriginal,0,0);

            webAppCore.saveScreenshot(exCanvas);

            setTimeout(function() {
                document.body.removeChild(exCanvas);
            },2000);
            
        };

        loadOverrideForCurrent();
    }

    function clearOverride() {
        document.getElementById("overrideImage").src = "";
        currentOverride = null;
        document.getElementById("overrideFrameCount").innerHTML = "";
    }

    function loadOverrideForCurrent() {

        if(!currentModFile) {
            startNewModFile();
        }

        if(currentAssetItem == null) {
            clearOverride();
            return;
        }

        if(!currentModFile.overrides[currentAssetItem.name]) {
            clearOverride();
            return;
        }

        var overrideData = currentModFile.overrides[currentAssetItem.name];

        if(overrideData.indexOf("<svg") > -1) {
            var svgBlob = new Blob([overrideData], {type: "image/svg+xml"});
            overrideData = window.URL.createObjectURL(svgBlob);
        }
        

        currentOverride = new Image();
        currentOverride.onload = function() {
            var frames = this.width / currentAssetItem.tileWidth;
            document.getElementById("overrideFrameCount").innerHTML = "Frames: " + frames;
        };
        currentOverride.src = overrideData;

        document.getElementById("overrideImage").src = overrideData;
        
    }

    function onSvgImport(name,content) {
        var svgBlob = new Blob([content], {type: "image/svg+xml"});
        var url = window.URL.createObjectURL(svgBlob);
        var img = new Image();

        // copy it to the canvas
        img.onload = function() {
            var frames = this.width / currentAssetItem.tileWidth;

            if(isFloat(frames)) {
                webAppCore.showAlert("Image is incorrect size.  Frame width must be equal to the original asset.");
            } else {
                currentModFile.overrides[currentAssetItem.name] = content;
                
                loadOverrideForCurrent();
                populateTextureList(false);
            }
        };

        img.addEventListener("error",function(){
            webAppCore.showToast("Error parsing .svg");
        });

        img.src = url;
    }

    function onImageImport(data) {
        if(currentAssetItem == null) {
            return;
        }

        var checkImage = new Image();
        checkImage.onload = function() {

            var frames = this.width / currentAssetItem.tileWidth;

            if(isFloat(frames)) {
                webAppCore.showAlert("Image is incorrect size.  Frame width must be equal to the original asset.");
            } else {
                

                // reconvert it because of windows blob imports

                var lCanvas = document.createElement("canvas");
                var lContext = lCanvas.getContext("2d");
                
                lCanvas.height = this.height;
                lCanvas.width = this.width;

                lContext.drawImage(this,0,0);

                var finalImageData = lCanvas.toDataURL();

                currentModFile.overrides[currentAssetItem.name] = finalImageData;
                
                loadOverrideForCurrent();
                populateTextureList(false);
            }

            
        };
        checkImage.src = data;
        

        
    }

    function isFloat(n) {
        return n === +n && n !== (n|0);
    }

    function placeholder() {}
    
    function saveMod(callback) {
        if(currentFileName == null) {
            if(currentModFile.meta.name.trim().length == 0) {
                webAppCore.showAlert("Please choose a name for this mod before saving.");
                return;
            } else {
                currentFileName = cleanFileName(currentModFile.meta.name.trim()) + "." + ColonyGame.meta.graphicsModFileFormat;
            }
        }

        currentModFile.meta.revision++;

        if(webAppCore.isApeAppsLoggedIn()) {
            currentModFile.meta.author = webAppCore.getApeAppsProfile().un;
        }

        webAppCore.storeVar("text-edit-last-save",currentFileName);
        webAppCore.storeVar("mod-name-ref-" + currentFileName,currentModFile.meta.name.trim());
        webAppCore.storeVar("mod-author-ref-" + currentFileName,currentModFile.meta.author.trim());

        setTimeout(function() {
            var saveContent = JSON.stringify(currentModFile);
            webAppCore.saveFile(currentFileName,saveContent,callback,true);
        },180);

        
    }

    function cleanFileName(name) {
        var clean = name.replace(/([^a-z0-9]+)/gi, "-");
        return clean.toLowerCase();
    }

    function onFileOpened(data) {

        if(data == null || data.trim().length == 0) {
            webAppCore.clearSplash();
            return;
        }

        try {
            currentModFile = JSON.parse(data);
            populateTextureList(true);
            populateMetaData();

            if(pendingOpenFileName != null) {
                currentFileName = pendingOpenFileName;
                webAppCore.storeVar("text-edit-last-save",pendingOpenFileName);
                pendingOpenFileName = null;
            }
        } catch(e) {
            console.log(e);
            currentModFile = null;
        }

        webAppCore.clearSplash();
    }
})();