var textureManager = textureManager || {};

(function(textureManager){

    var imageAssets = {};
    var tileSize = 0;
    var renderScaleFactor = 1;
    var defImageDirectory = "";
    var hardTileSize = 0;

    function initAssets() {
        imageAssets = {};
    }

    function setTileSize(size) {
        tileSize = size;
        imageAssets = {};
    }

    function setHardTileSize(size) {
        hardTileSize = size;
        imageAssets = {};
    }

    function setScaleFactor(factor) {
        renderScaleFactor = factor;
        imageAssets = {};
    }

    function setDefaultImageDirectory(dir) {
        defImageDirectory = dir;
    }

    function getFrameCount(img,color,flipped,outlineWidth,outlineColor) {
        if(!img) {
            return 0;
        }

        if(!color) {
            color = "def";
        }

        var flippedString = "noflip";

        if(flipped) {
            flippedString = "flip";
        } else {
            flipped = false;
        }

        if(!outlineWidth) {
            outlineWidth = 0;
        }

        if(!outlineColor) {
            outlineColor = "#000000";
        }

        if(!imageAssets[img]) {
            imageAssets[img] = {};
        }

        if(!imageAssets[img][color]) {
            imageAssets[img][color] = {};
        }

        if(!imageAssets[img][color][outlineColor]) {
            imageAssets[img][color][outlineColor] = {};
        }

        if(!imageAssets[img][color][outlineColor][outlineWidth]) {
            imageAssets[img][color][outlineColor][outlineWidth] = {};
        }

        if(imageAssets[img][color][outlineColor][outlineWidth][flippedString]) {
            if(imageAssets[img][color][outlineColor][outlineWidth][flippedString].loading) {
                return 0;
            } else {
                return imageAssets[img][color][outlineColor][outlineWidth][flippedString].frames.length;
            }
        }

        return 0;
    }

    function getAssetWithOptions(options) {
        var img = null;
        var frame = 0;
        var color = "def";
        var flipped = false;
        var custTileSize = tileSize;
        var outlineWidth = 0;
        var outlineColor = "";
        var withData = null;

        if(options.img) {
            img = options.img;
        } else {
            return null;
        }

        if(options.frame) {
            frame = options.frame;
        }

        if(options.color) {
            color = options.color;
        }

        if(options.flipped) {
            flipped = options.flipped;
        }

        if(options.custTileSize) {
            custTileSize = options.custTileSize;
        }

        if(options.outlineWidth) {
            outlineWidth = options.outlineWidth;
        }

        if(options.outlineColor) {
            outlineColor = options.outlineColor;
        }

        if(options.withData) {
            withData = options.withData;
        }


        return getAsset(img,frame,color,flipped,custTileSize,outlineWidth,outlineColor,withData);
    }

    function preloadAsset(img,frame,color,flipped,custTileSize,outlineWidth,outlineColor,withData) {
        if(!img) {
            return;
        }

        if(!custTileSize) {
            custTileSize = tileSize;
        }

        if(!frame) {
            frame = 0;
        }

        if(!color) {
            color = "def";
        }

        if(!outlineWidth) {
            outlineWidth = 0;
        }

        if(!outlineColor) {
            outlineColor = "#000000";
        }

        var flippedString = "noflip";

        if(flipped) {
            flippedString = "flip";
        } else {
            flipped = false;
        }

        if(!imageAssets[img]) {
            getAsset(img,frame,color,flipped,custTileSize,outlineWidth,outlineColor,withData);
            return;
        }

        if(!imageAssets[img][color]) {
            getAsset(img,frame,color,flipped,custTileSize,outlineWidth,outlineColor,withData);
            return;
        }

        if(!imageAssets[img][color][outlineColor]) {
            getAsset(img,frame,color,flipped,custTileSize,outlineWidth,outlineColor,withData);
            return;
        }

        if(!imageAssets[img][color][outlineColor][outlineWidth]) {
            getAsset(img,frame,color,flipped,custTileSize,outlineWidth,outlineColor,withData);
            return;
        }

        if(!imageAssets[img][color][outlineColor][outlineWidth][flippedString]) {
            getAsset(img,frame,color,flipped,custTileSize,outlineWidth,outlineColor,withData);
            return;
        }
    }

    function getAsset(img,frame,color,flipped,custTileSize,outlineWidth,outlineColor,withData) {
        if(!img) {
            return null;
        }

        if(!custTileSize) {
            custTileSize = tileSize;
        }

        if(!frame) {
            frame = 0;
        }

        if(!color) {
            color = "def";
        }

        if(!outlineWidth) {
            outlineWidth = 0;
        }

        if(!outlineColor) {
            outlineColor = "#000000";
        }

        var flippedString = "noflip";

        if(flipped) {
            flippedString = "flip";
        } else {
            flipped = false;
        }

        if(!imageAssets[img]) {
            imageAssets[img] = {};
        }

        if(!imageAssets[img][color]) {
            imageAssets[img][color] = {};
        }

        if(!imageAssets[img][color][outlineColor]) {
            imageAssets[img][color][outlineColor] = {};
        }

        if(!imageAssets[img][color][outlineColor][outlineWidth]) {
            imageAssets[img][color][outlineColor][outlineWidth] = {};
        }

        if(imageAssets[img][color][outlineColor][outlineWidth][flippedString]) {
            if(imageAssets[img][color][outlineColor][outlineWidth][flippedString].loading) {
                return null;
            } else {

                if(frame >= imageAssets[img][color][outlineColor][outlineWidth][flippedString].frames.length) {
                    frame = 0;
                }

                return imageAssets[img][color][outlineColor][outlineWidth][flippedString].frames[frame];
            }
        }

        var useSrc = img;


        if(img.indexOf("http:") == 0 || img.indexOf("https:") == 0 || img.indexOf("data:") == 0) {
            //preImage.crossOrigin= "anonymous";
            //console.log(img);
        } else {
            useSrc = defImageDirectory + img;

            if(withData && withData.indexOf("data:") == 0) {
                useSrc = withData;
            }
        }

        imageAssets[img][color][outlineColor][outlineWidth][flippedString] = {
            frames: [],
            loading: true
        };

        var issvg = endsWith(img,".svg");

        if(color == "def") {
            issvg = false;
        }

        if(withData && withData.toLowerCase().indexOf("<?xml") == 0) {
            issvg = true;
            processLoadedSvgText(withData,img,color,flipped,custTileSize,outlineColor,outlineWidth);
            return null;
        }

        if(issvg) {

            var xhr = new XMLHttpRequest();
            xhr.nameRef = img;
            xhr.colorRef = color;
            xhr.flipRef = flipped;
            xhr.olRef = outlineColor;
            xhr.olwRef = outlineWidth;
            xhr.srcRef = img;
            xhr.tSizeRef = custTileSize;
            xhr.onload = function() {
                var svgAsXml = xhr.responseXML;
                var svgAsString = new XMLSerializer().serializeToString(svgAsXml);
                processLoadedSvgText(svgAsString,this.srcRef,this.colorRef,this.flipRef,this.tSizeRef,this.olRef,this.olwRef);
            };
            xhr.addEventListener("error",function(){
                var flippedString = "noflip";

                if(this.flipRef) {
                    flippedString = "flip";
                } else {
                    this.flipRef = false;
                }

                delete imageAssets[this.srcRef][this.colorRef][this.olRef][this.olwRef][flippedString];
            });
            xhr.open("GET", useSrc);
            xhr.responseType = "document";
            xhr.send();
        } else {
            var preImage = new Image();
            if(img.indexOf("http:") == 0 || img.indexOf("https:") == 0 || img.indexOf("data:") == 0) {
                preImage.crossOrigin= "anonymous";
            }
            preImage.srcRef = img;
            preImage.colorRef = color;
            preImage.flipRef = flipped;
            preImage.olRef = outlineColor;
            preImage.olwRef = outlineWidth;
            preImage.tSizeRef = custTileSize;
            preImage.addEventListener("load",function() {
                frameizeImage(this,this.colorRef,this.flipRef,this.tSizeRef,this.olRef,this.olwRef);
            });
            preImage.addEventListener("error",function(e) {

                var flippedString = "noflip";
    
                if(this.flipRef) {
                    flippedString = "flip";
                } else {
                    this.flipRef = false;
                }
    
                delete imageAssets[this.srcRef][this.colorRef][this.olRef][this.olwRef][flippedString];
            });
    
            preImage.src = useSrc;
        }


        return null;
    }

    function processLoadedSvgText(svgAsString,srcRef,colorRef,flipRef,custTileSize,outlineColor,outlineWidth) {
        if(colorRef) {
            svgAsString = replaceAll(svgAsString, "fill:#ff00ff;", "fill:" + colorRef + ";");
        }

        // create a new image with the svg string as an ObjectUrl
        var svgBlob = new Blob([svgAsString], {type: "image/svg+xml"});
        var url = window.URL.createObjectURL(svgBlob);
        var img = new Image();
        img.tileReference = srcRef;
        img.nameRef = srcRef;
        img.colorRef = colorRef;
        img.flipRef = flipRef;
        img.srcRef = srcRef;
        img.olRef = outlineColor;
        img.olwRef = outlineWidth;
        img.tSizeRef = custTileSize;
        
        // copy it to the canvas
        img.onload = function() {
            frameizeImage(this,this.colorRef,this.flipRef,this.tSizeRef,this.olRef,this.olwRef);
            window.URL.revokeObjectURL(svgBlob);
        };

        img.addEventListener("error",function(){

            var flippedString = "noflip";

            if(this.flipRef) {
                flippedString = "flip";
            } else {
                this.flipRef = false;
            }

            delete imageAssets[this.srcRef][this.colorRef][this.olRef][this.olwRef][flippedString];
        });

        img.src = url;
    }

    function frameizeImage(img,color,flipped,custTileSize,outlineColor,outlineWidth) {
        var flippedString = "noflip";

        if(flipped) {
            flippedString = "flip";
        } else {
            flipped = false;
        }

        if(!img || !img.srcRef) {
            return;
        }

        if(!imageAssets[img.srcRef]) {
            imageAssets[img.srcRef] = {};
        }

        if(!imageAssets[img.srcRef][color]) {
            imageAssets[img.srcRef][color] = {};
        }

        if(!imageAssets[img.srcRef][color][outlineColor]) {
            imageAssets[img.srcRef][color][outlineColor] = {};
        }

        if(!imageAssets[img.srcRef][color][outlineColor][outlineWidth]) {
            imageAssets[img.srcRef][color][outlineColor][outlineWidth] = {};
        }

        if(!imageAssets[img.srcRef][color][outlineColor][outlineWidth][flippedString]) {
            imageAssets[img.srcRef][color][outlineColor][outlineWidth][flippedString] = {
                frames: [],
                loading: true
            };
        }

        var useScaleFactor = renderScaleFactor;

        if(hardTileSize && hardTileSize > 0) {
            useScaleFactor = hardTileSize / img.width;
        }
        
        var myHeight = img.height * useScaleFactor;

        imageAssets[img.srcRef][color][outlineColor][outlineWidth][flippedString].frames = [];

        var useTileSize = custTileSize;

        if(useTileSize == 0) {
            useTileSize = img.width;
        }

        var myWidth = useTileSize * useScaleFactor;

        var myFrames = img.width / useTileSize;

        var aCanvas = document.createElement("canvas");
        var aContext = aCanvas.getContext("2d");

        aCanvas.width = myWidth;
        aCanvas.height = myHeight;

        for(var i = 0; i < myFrames; i++) {
            aContext.clearRect(0,0,myWidth,myHeight);

            var drawX = 0;

            aContext.save();

            if(img.flipRef) {
                aContext.scale(-1,1);
                drawX = -myWidth;
            }

            
            aContext.drawImage(img,(i * useTileSize),0,useTileSize,img.height,drawX,0,myWidth,myHeight);

            aContext.restore();
            
            if(outlineWidth > 0) {
                var imgData = aContext.getImageData(0,0,aCanvas.width,aCanvas.height);
                var data = imgData.data;

                var defineNonTransparent = function (x, y) {

                    if(x <= 0 || y <= 0) {
                        return false;
                    }

                    var checkval = (y * aCanvas.width + x) * 4 + 3;

                    if(checkval >= data.length) {
                        return false;
                    }

                    return (data[checkval] > 0);
                };

                var points = geom.contour(defineNonTransparent,null,data,aCanvas.width);

                if(points.length > 0) {
                    aContext.strokeStyle = outlineColor;
                    aContext.lineWidth = parseFloat(outlineWidth);

                    aContext.beginPath();
                    aContext.moveTo(points[0][0],points[0][4]);
    
                    for(var p=1; p < points.length; p++){
                        var point = points[p];
                        aContext.lineTo(point[0],point[1]);
                    }
    
                    aContext.closePath();
                    aContext.stroke();
                }
                
            }
            

            var imFrame = new Image();
            imFrame.src = aCanvas.toDataURL();
            imageAssets[img.srcRef][color][outlineColor][outlineWidth][flippedString].frames.push(imFrame);
        }

        aContext = null;
        aCanvas = null;

        imageAssets[img.srcRef][color][outlineColor][outlineWidth][flippedString].loading = false;

        img = null;
    }

    function endsWith(str, suffix) {
        if(!str || !suffix) {
            return false;
        }

        return str.indexOf(suffix, str.length - suffix.length) !== -1;
    }

    function replaceAll(str, find, replace) {

        if(!str) {
            return "";
        }

        if(str.indexOf(find) < 0) {
            return str;
        }

        return str.replace(new RegExp(escapeRegExp(find), "g"), replace);
    }

    function escapeRegExp(str) {
        return str.replace(/([.*+?^=!:${}()|[]\/\\])/g, "\\$1");
    }

    // Marching Squares Edge Detection
    // source = d3-plugin for calculating outline paths - https://github.com/d3/d3-plugins/ - https://stackoverflow.com/questions/28207232/draw-border-around-nontransparent-part-of-image-on-canvas/28220510
    // modified*

    var geom = {};
    geom.contour = function(grid, start) { 
        var s = start || d3_geom_contourStart(grid), // starting point 
            c = [],    // contour polygon 
            x = s[0],  // current x position 
            y = s[1],  // current y position 
            dx = 0,    // next x direction 
            dy = 0,    // next y direction 
            pdx = NaN, // previous x direction 
            pdy = NaN, // previous y direction 
            i = 0;
        
        //console.log(grid);

        //console.log(x + ":" + y + " - " + dx + ":" + dy);

        do { 
            // determine marching squares index 
            i = 0; 
            if (grid(x-1, y-1)) i += 1; 
            if (grid(x,   y-1)) i += 2; 
            if (grid(x-1, y  )) i += 4; 
            if (grid(x,   y  )) i += 8; 

            // determine next direction 
            if (i === 6) { 
                dx = pdy === -1 ? -1 : 1; 
                dy = 0; 
            } else if (i === 9) { 
                dx = 0; 
                dy = pdx === 1 ? -1 : 1; 
            } else { 
                dx = d3_geom_contourDx[i]; 
                dy = d3_geom_contourDy[i]; 
            } 

            // update contour polygon 
            if (dx != pdx && dy != pdy) { 
                c.push([x, y]); 
                pdx = dx; 
                pdy = dy; 
            } 

            x += dx; 
            y += dy; 


            //console.log(x + ":" + y + " - " + dx + ":" + dy + " - " + i);
        } while (s[0] != x || s[1] != y); 

        return c; 
    };

    var d3_geom_contourDx = [1, 0, 1, 1,-1, 0,-1, 1,0, 0,0,0,-1, 0,-1,NaN], 
        d3_geom_contourDy = [0,-1, 0, 0, 0,-1, 0, 0,1,-1,1,1, 0,-1, 0,NaN];

    function d3_geom_contourStart(grid) { 
        var x = 0, 
            y = 0; 

        // search for a starting point; begin at origin 
        // and proceed along outward-expanding diagonals 
        while (true) { 
            if (grid(x,y)) { 
                return [x,y]; 
            } 
            if (x === 0) { 
                x = y + 1; 
                y = 0; 
            } else { 
                x = x - 1; 
                y = y + 1; 
            } 
        } 
    }



    textureManager.initAssets = initAssets;
    textureManager.setTileSize = setTileSize;
    textureManager.setScaleFactor = setScaleFactor;
    textureManager.getAsset = getAsset;
    textureManager.preloadAsset = preloadAsset;
    textureManager.setDefaultImageDirectory = setDefaultImageDirectory;
    textureManager.getFrameCount = getFrameCount;
    textureManager.setHardTileSize = setHardTileSize;
    textureManager.getAssetWithOptions = getAssetWithOptions;
})(textureManager);